if (!window.bridgeCore) {
  (() => {
    const CODE_SUCCESS = 200;
    const CODE_FAIL_100 = 100;
    const CODE_FAIL_101 = 101;
    const CODE_FAIL_102 = 102;
    const CODE_FAIL_103 = 103;
    const CODE_FAIL_110 = 110;
    const CODE_EXCEPTION = 500;

    //储存callbackId 和 对应的回调方法
    //客户端内部根据有无callbackId 判断同步或者异步方式调用
    const map = {};

    //同步调用
    //方法名字 methodName ui.progress
    //参数 数组类型 可不传递
    function callBridgeSync(name, p) {
      const request = {
        method: name,
        params: p
      };

      console.log("callBridgeSync", JSON.stringify(request));
      const businessData = analyzeResultException(
        window.a_bridge.call(JSON.stringify(request)),
        true
      );
      if (businessData == null) {
        console.log("callBridgeSync response null");
        return null;
      } else {
        console.log(
          "callBridgeSync response ",
          JSON.stringify(businessData.data)
        );
        return businessData.data;
      }
    }

    //异步调用
    //方法名字 methodName ui.progress
    //参数 数组类型
    //func 异步回调func
    function callBridge(name, p, func) {
      const request = {
        method: name,
        uniqueId: Date.now() * 1000000 + Math.round(Math.random() * 1000000),
        params: p
      };

      //callbackId 和 func map
      map[request.uniqueId] = func;

      console.log("callBridge", JSON.stringify(request));

      return analyzeResultException(
        window.a_bridge.call(JSON.stringify(request)),
        true
      );
    }

    //回调 js接受的地方
    function receiver(callbackId, keepCallback, result) {
      console.log("receiver", callbackId, keepCallback, result);
      //call
      map[callbackId](analyzeResultException(result), false);
      //delete
      if (!keepCallback) {
        delete map[callbackId];
      }
    }

    function analyzeResult(r) {
      let result;
      if (typeof r === "string") {
        result = JSON.parse(r);
      } else {
        result = r;
      }
      if (result === null) {
        return {
          code: -1,
          message: "no data",
          data: null
        };
      }

      let internalBCode = -1;
      let internalBMsg = "no msg";
      let internalBData = null;
      if (result.hasOwnProperty("bCode")) {
        internalBCode = result.bCode;
      }
      if (result.hasOwnProperty("bMsg")) {
        internalBMsg = result.bMsg;
      }
      if (result.hasOwnProperty("bData")) {
        internalBData = result.bData;
      }
      return {
        code: internalBCode,
        message: internalBMsg,
        data: internalBData
      };
    }

    function checkError(r) {
      console.log("checkError original", r);
      let result;
      if (typeof r === "string") {
        result = JSON.parse(r);
      } else {
        result = r;
      }
      if (result === null) {
        return {
          code: CODE_FAIL_100,
          message: "no result",
          data: null
        };
      }
      let internalCode;
      let internalMessage;
      let internalData;

      let hasCodeProperty = result.hasOwnProperty("code");
      if (hasCodeProperty) {
        internalCode = result.code;
      } else {
        internalCode = CODE_FAIL_100;
      }
      if (result.hasOwnProperty("message")) {
        internalMessage = result.message;
      } else {
        internalMessage = "The returned result does not contain message";
      }
      if (result.hasOwnProperty("data")) {
        internalData = result.data;
      } else {
        internalData = null;
      }
      return {
        code: internalCode,
        message: internalMessage,
        data: internalData
      };
    }

    function analyzeResultException(r, b) {
      const result = checkError(r);
      if (result.code == CODE_SUCCESS) {
        const businessData = analyzeResult(result.data);
        return businessData;
      }

      if (b) {
        throw result.message;
      } else {
        return result;
      }
    }

    //对外提供
    window.bridgeCore = {
      receiver: (callbackId, keepCallback, result) => {
        receiver(callbackId, keepCallback, result);
      },

      callBridgeSync: (name, p) => {
        return callBridgeSync(name, p);
      },

      callBridge: (name, p, func) => {
        return callBridge(name, p, func);
      }
    };
  })();
}
